#!/usr/bin/perl

#use lib "/home/j/clp/chinese/tools/lib/perl";
use TBNode;

%indices; #need to be refreshed for each sentence
$index; #need to be refreshed for each sentence
%labels; #need to be refreshed for each sentence
$basefilename = "";


my $outputdir = $ARGV[0];
$outputdir ne "" || die "Empty output directory\n";
if(-e "$outputdir"){
    print "Find output in <$outputdir>\n";
}else{
    print "Creating <$outputdir> directory...";
    `mkdir $outputdir`;
}

foreach my $index (1..$#ARGV){
    my $file = $ARGV[$index];
    print STDERR "Processing file $file\n";
    my $basefile = `basename $file`;
    chomp($basefile);
    open(OUT, ">$outputdir/$basefile.head");
    `cat $file | ./stripsgml.pl > /tmp/$basefile.$$`; 
    processfile("/tmp/$basefile.$$", \*OUT);
    `cat $outputdir/$basefile.head | ./indent.pl > $outputdir/$basefile.head.idt`;
}


sub processfile{
    my ($file, $outfh) = @_;
    open(F, $file);
    $sentcount=0;  # reset sentence count for each file
    my $fh = \*F;
    while (defined($tbn = TBNode::nextSentence($fh))) {
	#do stuff with a tbnode object $tbn ...
	my $firstchild = $tbn->firstChild();
	#print "Sentence No. $sentcount\n";
	if($firstchild){
	    print $outfh "(", $firstchild->label(), " ";
	    undef %indices; #reset the indices hash for this sentence
	    undef $labels; #reset the labels hash for this sentence
	    $index = 400;
	    &generate_new_indices($firstchild);
	    foreach my $k (sort (keys %indices)){
		#print $outfh "indices ", $k, "\t", $indices{$k}, "\t", $labels{$k}, "\n";
	    }

	    &process_node($outfh, $firstchild);
	    print $outfh ")\n";
	}
	$sentcount++;
    }
    close(F);
}

sub get_head_and_relation{ 
    #determine if the structure is one of adjunction (adj), 
    #complementation (comp), auxiliary (aux), coordination (coord), predication (pred), or flat
    #return values: "adj", "comp", "aux", "coord", "pred", "flat" as well as the head
    #of this constituent
    my ($node) = @_;
    my $nodeLabel = $node->label();
    my @children = $node->children();
    my $nonterm_head = ""; #non-terminal head, head-final 
    my $term_head = ""; #terminal head
    my $CCs = 0;
    my $PUs = 0;
    my $terminals = 0; #number of terminal chidren in constituent that are NOT punctuation marks
    my $type = "flat"; #default relation is flat
    my $allterminal = 1; #a boolean to determine if all the children are terminals
    my %allnonterms;
    my %allterminals;
    my $maxNonterm = 0; #maximum number of nonterms that have the same label
    my $uniqlabels = 0; #number of unique labels in the non-terminal children
    my $uniqtermlabels = 0;

    foreach my $child (@children){
	if($child->height() == 0){
	    if($child->label() eq "CC"){
		$CCs++;
	    }elsif($child->label() eq "ETC"){
		$CCs++;
	    }elsif($child->label() eq "PU"){
		$PUs++;
	    }elsif($child->label() eq "AS"){
		#don't update the terminal head
	    }else{
		$terminals++;
		$term_head = $child;
		$allterminals{$child->label()}++;
	    }
	}elsif(isCompound($child)){
	    #treat compounds as terminals
	    $term_head = $child;
	}else{
	    if($child->label() ne "PRN"){
		$allterminal = 0;
		$allnonterms{$child->label()}++;
		$nonterm_head = $child;
	    }
	}
    }

    foreach my $k (keys %allterminals){
	$uniqtermlabels++;
    }


    foreach my $k (keys %allnonterms){
	$uniqlabels++;
	if($allnonterms{$k} > $maxNonterm){
	    $maxNonterm = $allnonterms{$k};
	}
    }


    #the last non-punctuation terminal is CC
    if($CCs || #if there is a CC
       $nodeLabel eq "VCD"||
       ($terminals > 1 && $allterminal && $PUs > 0 && $uniqtermlabels == 1)||
       (!$term_head && $maxNonterm >1 && $PUs > 0 && $uniqlabels == 1) || #or no terminals other than PU
                                            #there are two or more nonterms with the same label  
       (!$term_head && $nodeLabel eq "UCP")){
	if($nonterm_head){ 
	    my $headaddr = createID($nonterm_head);
	    return "coord|$headaddr";
	}elsif($term_head){
	    my $headaddr = createID($term_head);
	    return "coord|$headaddr";
	}else{
	    return "coord|unspecified"; #do not specify a head, e.g., conjunctions
	}
    }


    #the structure is all flat
    if($allterminal){
	if ($nodeLabel eq "VRD" || $nodeLabel eq "VCD" || $nodeLabel eq "VCP" || $nodeLabel eq "VNV" || $nodeLabel eq "VPT"){
	    my $headaddr = createID($children[0]);
	    return "flat|$headaddr";
	}elsif($term_head){  
	    my $headaddr = createID($term_head);
	    return "flat|$headaddr";
	}else{
	    #the head is empty because there is only PU or CC
	    #use the first child
	    my $headaddr = createID($children[0]);
	    return "flat|$headaddr";
	}
    }


    #if the last non-terminal structure is VC or VV and
    #its right sibling is a VP, this is an auxiliary
    #head is the VP
    if($term_head && $term_head->nextSibling() && $term_head->nextSibling()->label() eq "VP"){
	my $headaddr = createID($term_head->nextSibling()); 
	return "aux|$headaddr";
    }

    #if there is a terminal, it's comp
    if($term_head){
	my $headaddr = createID($term_head);
	return "comp|$headaddr";
    }

    #if this is an IP node and the headnode is a VP, then 
    #this is a predication
    if(!$term_head && $nodeLabel =~ /^IP/ && $nonterm_head->label() eq "VP"){
	my $headaddr = createID($nonterm_head);
	return "pred|$headaddr";
    }

    #everything else is treated as adjunction and the headnode is the last non-punctuation node
    my $headaddr = createID($nonterm_head);
    return "adj|$headaddr";
}


sub generate_new_indices{
    my ($node) = @_;
    my @children = $node->children();
    foreach my $child (@children){
	my $key = $child->termno().":".$child->height();
	$labels{$key} = $child->label();
	if(isLeaf($child)){
	    $indices{$key} = $child->termno();
	}else{
	    $indices{$key} = $index;
	    $index++;
	}
	if(!isLeaf($child)){
	    generate_new_indices($child);
	}
    }
}

sub printRawLabel{
    #print the label of a node
    my ($outfh, $primary, $ftag, $index) = @_;
    print $outfh "(", $primary;
    if ($ftag){
	print $outfh $ftag;
    }
    if ($index){
	print $outfh "-", $index;
    }
    print $outfh " ";
}

sub process_node{
    
    my $outfh = shift;
    my @nodes = @_;  #the first argument is the node in question, other arguments are optional
                     #and contain auxiliaries, if there are any.
    my $node = $nodes[0];

    if(isLeaf($node)){ #this is where to add auxiliaries for verbs, if there are any 
	print $outfh $node->rawdata();
    }else{
	my $nodeLabel = $node->label();
	my $relation_headaddr = get_head_and_relation($node);
	my ($relation, $headaddr) = split(/\|/, $relation_headaddr);
	my @children = $node->children();

	if($nodeLabel eq "IP"){
	    if($relation eq "pred"){
		foreach my $child (@children){
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "ADV";} #default functional tag
		    if($headaddr eq $child->termno().":".$child->height()){
			printRawLabel($outfh, $child->label(), "-HEAD", "");
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			printRawLabel($outfh, $child->label(), "", "");		
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			$conjuncts++;
			printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "ADV";} #default functional tag

		    if($headaddr eq $child->termno().":".$child->height()){
			printRawLabel($outfh, $childlabel, "-HEAD", "");		
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{

		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
			#http://w3.msi.vxu.se/~nivre/research/chn_headrules.txt
		    }
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "ADV";} #default functional tag

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "VP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "COMP";} #default functional tag
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "AS"){
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			$conjuncts++;
			printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "OTHER";} #default functional tag
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "AS"){
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "aux"){
		foreach my $child (@children){
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "AUX";} #default functional tag
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    my $ftag = getFtag($child);
		    if($ftag eq "NONE") {$ftag = "ADV";} #default functional tag
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{

		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "VRD"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[0])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "VSB"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "VNV"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[0])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "VCD"){
	    if($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif(isVerb($child)){
			$conjuncts++;
			print $outfh "(", $child->label(), "-CONJUNCT$conjuncts ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			$conjuncts++;
			print $outfh "(", $child->label(), "-CONJUNCT$conjuncts ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		my $firstchild = $children[0];
		#reset the headaddr
		$headaddr = $firstchild->termno().":".$firstchild->height();
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[0])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "VCP"){
	    if($relation eq "flat"){
		my $firstchild = $children[0];
		#reset the headaddr
		$headaddr = $firstchild->termno().":".$firstchild->height();
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[0])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "VPT"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-PRT ";
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[0])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "NP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		for (my $i=0; $i<=$#children; $i++){
		    my $child = $children[$i];
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			$conjuncts++;
			if(isLeaf($child)){
			    #add structures for flat NP coordination structures
			    print $outfh "(NP-CONJUNCT$conjuncts ";
			    my $count = 0; #count the number of nodes
			    my $m = $i;
			    my $ch = $child;
			    while(defined($ch) && isLeaf($ch) && !isConj($ch)){
				$count++;
				$m++;
				$ch = $children[$m];
			    }
			    my $n = 0;
			    while(defined($child) && isLeaf($child) && !isConj($child)){
				$n++;
				if($n == $count){
				    print $outfh "(", $child->label(), "-HEAD ";
				}else{
				    print $outfh "(", $child->label(), "-NMOD ";
				}
				process_node($outfh, $child);
				#printIndex($child, $outfh);
				print $outfh ")";
				$i++;
				$child = $children[$i];
			    }
			    #print $outfh "(index $index)";
			    #$index++;
			    $i--; #come back by one
			    print $outfh ")";
			}else{
			    printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			    process_node($outfh, $child);
			    #printIndex($child, $outfh);
			    print $outfh ")";
			}
		    }
		}

		#printConj($outfh);
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "DT"){
			print $outfh "(", $child->label(), "-DMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "JJ"){
			print $outfh "(", $child->label(), "-AMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "NN" || $child->label() eq "NR" || $child->label() eq "NT"){
			print $outfh "(", $child->label(), "-NMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    my $ftag=getFtag($child);
		    if($ftag eq "NONE"){$ftag = "UNSPEC";}
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "DP" || $child->label() eq "QP"){
			print $outfh "(", $child->label(), "-DMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "CP"){
			print $outfh "(", $child->label(), "-RELATIVE ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "IP"){
			print $outfh "(", $child->label(), "-APP ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "ADJP"){
			print $outfh "(", $child->label(), "-AMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PP"){
			print $outfh "(", $child->label(), "-AMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "NP" || $child->label() eq "DNP"){
			print $outfh "(", $child->label(), "-NMOD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-$ftag", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{

			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		

	    }
	}elsif($nodeLabel eq "UCP"){
	    if($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			$conjuncts++;
			print $outfh "(", $child->label(), "-CONJUNCT$conjuncts ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $childlabel, "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	    #printConj($outfh);
	}elsif($nodeLabel eq "PP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "NP"){
			printRawLabel($outfh, $child->label(), "-OBJ", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-COMP ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label();
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{

		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "DP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "NP"){
			printRawLabel($outfh, $child->label(), "-OBJ", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }		
	}elsif($nodeLabel eq "QP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			$conjuncts++;
			printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		

	    }
	}elsif($nodeLabel eq "CLP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{

		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "WHNP"){
	    foreach my $child (@children){
		print $outfh "(", $child->rawLabel(), "-HEAD ";
		process_node($outfh, $child);
		#printIndex($child, $outfh);
		print $outfh ")";
	    }
	}elsif($nodeLabel eq "WHPP"){
	    foreach my $child (@children){
		print $outfh "(", $child->rawLabel(), "-HEAD ";
		process_node($outfh, $child);
		#printIndex($child, $outfh);
		print $outfh ")";
	    }
	}elsif($nodeLabel eq "ADJP"){
	    if($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		my $conjunction = 0;
		my $conjuncts = 0;
		foreach my $child (@children){
		    if(isConj($child)){
			$conjunction++;
			print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			$conjuncts++;
			printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "ADVP"){
	    if($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "coord"){
		if($#children == 0){ #there is only one child, make it the head
		    my $child = $children[0];
		    print $outfh "(", $child->label(), "-HEAD ";
		    process_node($outfh, $child);
		    #printIndex($child, $outfh);
		    print $outfh ")";
		}else{
		    my $conjunction = 0;
		    my $conjuncts = 0;
		    foreach my $child (@children){
			if(isConj($child)){
			    $conjunction++;
			    print $outfh "(", $child->label(), "-CONJUNCTION$conjunction ";
			    process_node($outfh, $child);
			    #printIndex($child, $outfh);
			    print $outfh ")";
			}elsif($child->label() eq "PU"){
			    print $outfh "(", $child->label(), " ";
			    process_node($outfh, $child);
			    #printIndex($child, $outfh);
			    print $outfh ")";
			}else{
			    $conjuncts++;
			    printRawLabel($outfh, $child->label(), "-CONJUNCT$conjuncts", $child->traceId());		
			    process_node($outfh, $child);
			    #printIndex($child, $outfh);
			    print $outfh ")";
			}
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "CP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-SUBORD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label()," ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			#print "test ", $child->traceId(), " test\n";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}

	    }
	}elsif($nodeLabel eq "LCP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }
	}elsif($nodeLabel eq "DNP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-COMP", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{

		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $childlabel, "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "DVP"){
	    if($relation eq "comp"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-COMP ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $childlabel, "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}elsif($nodeLabel eq "PRN"){
	    if($relation eq "adj"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }elsif($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }elsif($child->label() eq "PU"){
			print $outfh "(", $child->label(), " ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-ADV", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			printRawLabel($outfh, $child->label(), "-UNSPEC", $child->traceId());		
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		

	    }
	}elsif($nodeLabel eq "FRAG"){
	    if($relation eq "flat"){
		foreach my $child (@children){
		    if($headaddr eq $child->termno().":".$child->height()){
			print $outfh "(", $child->label(), "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $child->label(), "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}
	    }else{
		foreach my $child (@children){
		    my $childlabel = $child->label();
		    if(!$childlabel){
			$childlabel =$child->rawLabel();
		    }

		    if(sameNode($child, $children[$#children])){
			print $outfh "(", $childlabel, "-HEAD ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }else{
			print $outfh "(", $childlabel, "-UNSPEC ";
			process_node($outfh, $child);
			#printIndex($child, $outfh);
			print $outfh ")";
		    }
		}		
	    }
	}else{

	    foreach my $child (@children){
		if(sameNode($child, $children[$#children])){
		    print $outfh "(", $child->label(), "-HEAD ";
		    process_node($outfh, $child);
		    #printIndex($child, $outfh);
		    print $outfh ")";
		}else{
		    print $outfh "(", $child->label(), "-UNSPEC ";
		    process_node($outfh, $child);
		    #printIndex($child, $outfh);
		    print $outfh ")";
		}
	    }
	}
    }
}






sub isLeaf{
    my ($node) = @_;
    if($node->height() == 0){
	return 1;
    }
    return 0;
}

sub isConj{
    my ($node) = @_;
    if($node->height() == 0 && 
       ($node->label() eq "CC" || 
	$node->label() eq "ETC" || 
	($node->label() eq "PU" &&
	 $node->data() ne ""))){
	return 1;
    }
    return 0;
}

sub isCompound{
    my ($node) = @_;
    if($node->label() eq "VCD" || 
       $node->label() eq "VRD" || 
       $node->label() eq "VNV" || 
       $node->label() eq "VSB" || 
       $node->label() eq "VPT" || 
       $node->label() eq "VCP"){
	return 1;
    }
    return 0;
}


sub isFlat{
    my (@nodes) = @_;
    my $isflat = 1;
    foreach my $node (@nodes){
	if($node->height() != 0){
	    $isflat = 0;
	}
    }
    return $isflat;
}



sub isVerb{
    my ($node) = @_;
    if($node->label() =~ /^V/){
	return 1;
    }
    return 0;
}



sub printnode{
    my ($cnode, $outfh) = @_;
    print $outfh $cnode -> rawdata();
}


sub printConj{
    my ($outfh) = @_;
    print $outfh "(CONJOINED T)";
}


sub sameNode{
    my ($node1, $node2) =@_;
    return $node1->termno() == $node2->termno() && $node1->height() == $node2-> height();
}

sub getFtag{
    #filtering out the irrelevant tags
    my ($node) = @_;
    #put the nodes in a hash
    my @ftags = $node->ftags();
    foreach my $ftag (@ftags){
	if($ftag eq "ADV"){
	    return "ADV";
	}elsif($ftag eq "APP"){
	    return "APP";
	}elsif($ftag eq "BNF"){
	    return "BNF";
	}elsif($ftag eq "CND"){
	    return "CND";
	}elsif($ftag eq "DIR"){
	    return "DIR";
	}elsif($ftag eq "EXT"){
	    return "EXT";
	}elsif($ftag eq "FOC"){
	    return "FOC";
	}elsif($ftag eq "IO"){
	    return "IO";
	}elsif($ftag eq "LGS"){
	    return "LGS";
	}elsif($ftag eq "LOC"){
	    return "LOC";
	}elsif($ftag eq "MNR"){
	    return "MNR";
	}elsif($ftag eq "PRD"){
	    return "PRD";
	}elsif($ftag eq "PRP"){
	    return "PRP";
	}elsif($ftag eq "SBJ"){
	    return "SBJ";
	}elsif($ftag eq "TMP"){
	    return "TMP";
	}elsif($ftag eq "TPC"){
	    return "TPC";
	}elsif($ftag eq "VOC"){
	    return "VOC";
	}
    }
    return "NONE";
}

sub printSemFeature{
    my ($outfh, $node) = @_;
    my @ftags = $node->ftags();
    my $ftag = $ftags[0];
    if(defined($ftag)){
	if($ftag eq "MNR"){
	    print $outfh "(SEM-FEATURE MNR)";
	}elsif($ftag eq "CND"){
	    print $outfh "(SEM-FEATURE CONDITION)";
	}elsif($ftag eq "DIR"){
	    print $outfh "(SEM-FEATURE DIR)";
	}elsif($ftag eq "PRP"){
	    print $outfh "(SEM-FEATURE PRP)";
	}elsif($ftag eq "EXT"){
	    print $outfh "(SEM-FEATURE EXT)";
	}elsif($ftag eq "Q"){
	    print $outfh "(MOOD INTERROGATIVE)";
	}elsif($ftag eq "FOC"){
	    print $outfh "(FOCUS T)";
	}elsif($ftag eq "HLN"){
	    print $outfh "(GENRE-TYPE HLN)";
	}elsif($ftag eq "TMP"){
	    print $outfh "(SEM-FEATURE TMP)";
	}elsif($ftag eq "IMP"){
	    print $outfh "(MOOD IMPERATIVE)";
	}elsif($ftag eq "TTL"){
	    print $outfh "(GENRE-TYPE TTL)";
	}elsif($ftag eq "LOC"){
	    print $outfh "(SEMFEATURE LOC)";
	}
    }

}

sub createID{
    my ($node) = @_;
    if(defined($node)){
	my $termno = $node->termno();
	my $height = $node->height();
	my $id = $termno . ":" . $height;
	return $id;
    }else{
	print "Error at $file $sentence\n";
    }
    return null; 
}

